use strict;
use Carp;

use vars qw(%in %text $tb $cb);

# Misc html goodies
#

my $noindent = 0;
my $indent = 0;

# Level 0: ancillary functions
sub iprint {
  my $text = shift(@_);
  my $ind="";

  if ($noindent == 0) {
    $ind =  "  " x $indent ;
  }
  print $ind.$text;
}


sub printTag {
  $| = 0;
  my $tagName = shift;
  my $body = shift;


  my $out = "<$tagName";
  foreach my $prop (@_) {
    if(! ref($prop)) {
      $out .= "$prop is not a reference\n";
      $out .= join(" | ", caller(0))."\n";
      $out .= join(" | ", caller(1))."\n";
      $out .= join(" | ", caller(2))."\n";
      $out .= join(" | ", caller(3))."\n";
      $out .= join(" | ", caller(4))."\n";
      $out .= join(" | ", caller(5))."\n";
      $out .= join(" | ", caller(6))."\n";
    }
    foreach my $key (keys %$prop) {
      $out .= " $key";
      my $value = $prop->{$key};
      if ($key eq "name") {
	$out .= "=\"$value\"";
      } elsif ($value ne "") {
	$out .= "=\"".&html_escape($value)."\"";
      }
    }
  }
  $out .= ">\n";
  iprint $out;
  if (defined $body) {
    $indent++;
    if (ref $body) {
      &$body();
    } else {
      &iprint ($body);
      if ($noindent == 0) {
	print "\n";
      }
    }
    $indent--;
    iprint "</$tagName>\n";
  }
}

# Level 1: individual HTML elements
# structural elements take functions as parameter, non-structural elements
# take strings.
sub printInput {
    my $type = shift;

    &printTag("input", undef, {	type => $type }, @_);
}

sub hidden($$) {
  my ($name, $value) = @_;
  &printInput("hidden", {name => $name, value => $value});
}


sub submit(@) {
    my $name = shift;
    if ($name eq "") {
	$name="$text{button_submit}";
    }
    &printInput("submit", { name => 'action', value => $name }, @_);
}

sub namedSubmit(@) {
    my $name = shift;
    my $label = shift;
    if ($label eq "") {
	$label="$text{button_submit}";
    }
    &printInput("submit", { name => $name, value => $label }, @_);
}

sub form($%@) {
    my $action = shift;
    my $body = shift;

    &printTag("form", $body, {
	action => "$action.cgi",
	method => 'post' }, @_);
}

sub table {
    my $body = shift(@_);
    if (ref $body eq "ARRAY") {
	&table(sub {
		 my $linex;
		 my $params="";

		 foreach my $linex (@$body) {
		   if (ref $linex eq "HASH") {
		     $params=$linex;
		   } else {
		     if ($params ne "") {
		       &tableRow($linex, $params);
		     } else {
		       &tableRow($linex);
		     }
		   }
		 }
	       }, @_);
      } elsif (ref $body eq "HASH") {
	&table(@_, $body);
    } elsif (ref $body eq "CODE" || not ref $body) {
	&printTag("table", $body, @_);
    } else {
	print STDERR "problem: ".join(" ",caller(0))."\n";
    }
}

sub tableRow(&@) {
    my $body = shift(@_);
    if (ref $body eq "ARRAY") {
	&tableRow(sub {
	    my $linex;
	    my $params="";

	    foreach my $linex (@$body) {
		if (ref $linex eq "HASH") {
		    $params=$linex;
		} else {
		    if ($params ne "") {
			&tableCell($linex, $params);
		    } else {
			&tableCell($linex);
		    }
		}
	    }
	}, @_);
    } else {
	&printTag("tr", $body, @_);
    }
}

sub tableCell(&@) {
    my $body = shift(@_);

    if (ref($body) eq "ARRAY") {
	tableCell {&table($body)}, @_;
    } else {
	&printTag("td", nz($body), @_);
    }
};


sub printCheckbox($$) {
    my $name = shift(@_);
    my $value = shift(@_);
    my %bool;

    if ($value ne "" && $value ne "0" && $value ne "no"){
	$bool{"checked"} = "";
    }

    &printInput("checkbox", { name => $name, value => 1}, \%bool, @_);
}


sub printRadio($$$) {
    my $name = shift(@_);
    my $choice = shift(@_);
    my $value = shift(@_);
    my %bool;

    if ($value ne "" && $value eq $choice){
	$bool{"checked"} = "";
    }

    &printInput("radio", { name => $name, value => $choice}, \%bool, @_);
}


sub printTextLine {
    my $name = shift(@_);
    my $value = shift(@_);

    &printInput("text", {name => $name, value => $value }, @_);
}

sub printFileInput {
    my $name = shift;

    &printInput("file", {name => $name }, @_);
}


sub printPasswordInput($$@) {
    my $name = shift;
    my $value = shift;
    &printInput("password",
		{name=>$name, value=>$value, size=>12}, @_);
}


sub printTextArea {
    my $name = shift(@_);
    my $value = shift(@_)."";

    $noindent=1;
    &printTag("textarea", &html_escape($value), {name => $name}, @_);
    $noindent=0;
}

sub printLink($$;$) {
    my $address= shift(@_).".cgi";
    my $text = shift(@_);
    my $assocs = shift(@_);
    my $first = 1;
    my $key;

    foreach my $key (keys %$assocs) {
	if ($first) {
	    $address .= "?";
	} else {
	    $address .= "&";
	}
	$first=0;
	$address .= "$key=".&urlize($assocs->{$key});
    }

    &printTag("a", $text, { href => $address });
}

sub printRedirect($;$) {
    my $address = shift(@_).".cgi";
    my $assocs = shift(@_);
    my $first = 1;
    my $key;

    foreach my $key (keys %$assocs) {
	if ($first) {
	    $address .= "?";
	} else {
	    $address .= "&";
	}
	$first=0;
	$address .= "$key=".&urlize($assocs->{$key});
    }

    &redirect($address);
}


sub printSelectionList($$$) {
    my $name = shift(@_);
    my $value = shift(@_);
    my $choiceList = shift(@_);
    my $linex;

    printTag("select", sub {
	       foreach my $linex (@{$choiceList}) {
		 iprint "<option value=\"$linex->[0]\"";
		 if ($value eq $linex->[0]) {
		   print " selected";
		 }
		 print ">$linex->[1]\n";
	       }}, { name => $name} );
}


sub printMultiList {
    my $name = shift(@_);
    my $choices = shift(@_);
    my $values = shift(@_);

    my %bitmap;
    my $val;

    foreach my $val (@$values) {
      $bitmap{$val}=1;
    }

    printTag("select", sub {
	       foreach my $elt (@$choices) {
		 iprint "<option value=$elt";
		 if ($bitmap{$elt}) {
		   print " selected";
		 }
		 print ">$elt\n";
	       }}, { name => $name,
		     multiple => undef,
		     size => 10 });
}


sub nbsp() {
    print "&nbsp;";
}

sub hr() {
    print "<hr>";
}

sub br() {
    print "<br>";
}

sub p() {
    print "<p>";
}

sub li() {
    print "<li>";
}

sub ul(&) {
    printTag("ul", @_);
}

sub center(&) {
    printTag("center", @_);
}

sub b($) {
    printTag("b", @_);
}

sub big($) {
    my ($body) = @_;
    printTag("font", $body, {size=>"+1"});
}

# Level 3: Combination of individual elements
sub headRow($$@) {
    my $body = shift(@_);
    my $width = shift(@_);

    tableRow {
	tableCell {iprint "<b>$body</b>\n"}, {colspan => $width};
    } ({ $tb }), @_;
}


sub anyRow(&@) {
    my $body = shift(@_);
    &tableRow($body, { $cb => ''}, @_);
}


sub mainTable(@) {
    my $title = shift(@_);
    my @args = @_;

    table sub {
	headRow ($title, 1);
	anyRow sub {
	    tableCell { &table(@args) }
	}},
    { 'border' }
}

sub mainForm($$%@) {
    my $action = shift(@_);
    my @args = @_;

    form ($action, sub { mainTable(@args) });
}

sub submitLine(@) {
    my $span = shift(@_);
    my @args = @_;
    if (!defined $span) {
	$span = 2;
    }
    tableCell { submit(@args) } { colspan => $span, align => "center"};
}

sub dumpIn() {
  for my $name (keys %in) {
    &hidden($name, $in{$name});
  }
}

sub dumpLogin() {
    &hidden('login', $in{'login'});
}


sub printLabel($) {
  my $label = shift(@_);
  &tableCell("$label", {align=>'right'});
}


sub printLabeledCheckbox($;$@) {
    my $label = shift(@_);
    my $name = shift(@_);
    my $value = shift(@_);
    my @args = @_;

    if (!defined $name) {
	$name = $label;
    }

    printLabel($label);
    tableCell { &printCheckbox($name, $value, @args) };
    &hidden("old.$name", $value);
}

sub printLabeledTextLine($;$@) {
    my $label = shift(@_);
    my $name = shift(@_);
    my $value = shift(@_);
    my @args = @_;

    if (!defined $name) {
	$name = $label;
    }

    printLabel($label);
    tableCell {&printTextLine($name, $value, @args)};
    &hidden("old.$name", $value);
}



sub printLabeledFileInput($;$@) {
    my $label = shift;
    my $name = shift;
    my @args = @_;

    if (!defined $name) {
	$name = $label;
    }
    printLabel($label);
    tableCell {&printFileInput($name, @args)};
}


sub printLabeledMultiList($;$@) {
    my $label = shift(@_);
    my $name = shift(@_);
    my @args = @_;

    if (!defined $name) {
	$name = $label;
    }
    printLabel($label);
    tableCell {&printMultiList($name, @args)};
}


sub printLabeledPasswordInput($;$@) {
    my $label = shift(@_);
    my $name = shift(@_);
    my @args = @_;

    if (!defined $name) {
	$name = $label;
    }
    printLabel($label);
    tableCell {&printPasswordInput($name, @args)};
}

sub nz($) {
  my ($value) = shift(@_);
  if($value =~ /^\s*$/ms) {
    return "&nbsp;";
  } else {
    return $value;
  }
}

1;
