use strict;

sub getByte {
  my ($digit) = @_;
  if ($digit =~ /[^0-9]/) {
    &error(&text('ip_nan', $digit));
  }
  if ($digit > 255) {
    &error(&text('ip_1byte', $digit));
  }
  return $digit;
}


sub getWord {
  my ($digit) = @_;
  if ($digit =~ /[^0-9]/) {
    &error(&text('ip_nan', $digit));
  }
  if ($digit > 65535) {
    &error(&text('ip_2bytes', $digit));
  }
  return $digit;
}


sub checkIpForValidity {
  my ($adr) = @_;
  if($adr =~ /^IP@/) {
    return 0;
  }
  my @adr = split(/\./, $adr);
  if (@adr != 4) {
    &error(&text('ip_badNrDigits', $adr));
  }
  my $ret=0;
  foreach my $digit (@adr) {
    $ret *= 256;
    $ret += getByte($digit);
  }
  return $ret;
}


sub checkNetworkForValidity {
  my ($line) = @_;
  my $base;
  my $netmask;
  if ($line =~ /^(.*)\/(.*)$/) {
    my $rip = $1;
    my $rmask = $2;

    my $ip = checkIpForValidity($rip);
    if ($rmask =~ /\./) {
      my $mask = checkIpForValidity($rmask);
      if (($mask | ($mask - 1)) != 0xffffffff) {
	&error(&text('ip_badMask', $rmask));
      }
    } else {
      if(getByte($rmask) > 32) {
	&error(&text('ip_badMask', $rmask));
      }
    }
  } else {
    checkIpForValidity($line);
  }
}

sub checkPortForValidity {
  my ($port) = @_;

  if ($port !~ /^([0-9]*)$/ || $port > 65535) {
      &error(&text('ip_badPort',$port));
  }
}


sub checkPortSequenceForValidity {
  my ($line) = @_;
  my $range;

  foreach my $range (split(" ", $line)) {
    if ($range =~ /^([0-9]*):([0-9]*)$/) {
      my $begin = $1;
      my $end = $2;
      if (getWord($begin) > getWord($end)) {
	&error(&text('ip_bad_range',$begin, $end, $line));
      }
    } else {
      getWord($range);
    }
  }
}


sub printIp {
  my ($adr) = @_;
  my $i;
  my @array;

  for(my $i=3; $i >=0; $i--) {
    $array[$i] = $adr % 256;
    $adr = $adr / 256;
  }
  return join(".", @array);
}


sub convertRangeToNets {
  my ($start, $end) = @_;
  my @adrs;
  my ($shift, $mask);

  $start=checkIpForValidity($in{'start'});
  $end=checkIpForValidity($in{'end'});

  if($start > $end) {
    &error(&text('ip_bad_range',$start, $end, "$in{start}-$in{end}"));
  }

  $end++;

  my @adrs;
  $shift = 1;
  $mask=0xffffffff;
  while($start < $end) {
    $mask = (1 << (32 - $shift)) - 1;
	if (($start & $mask) == 0 && ($start + $mask + 1) <= $end) {
	    push(@adrs, &printIp($start)."/$shift");
	    $start += $mask+1;
	    $shift = 1;
	} else {
	    $shift++;
	}
    }

    return join(" ", @adrs);
}

1;
