# This file contains utility methods for listfiles
# A listfile is a file containing a list of names (usernames, groupnames, etc.)
# It may contain comments
# Order is not important


BEGIN {
    require 'school-lib.pl';
}

use UserList;
use strict;
use English;
use Carp qw/croak/;
use vars qw/%text $LLL_SECURITY_DIR/;

require 'html.pl';

$LLL_SECURITY_DIR = "/etc/lll/security";

my $validChars = "[a-zA-Z0-9_-]";

# Read the namelist
sub getNamelist {
  my ($filename) = @_;
  my $list = new UserList;
  if(! -f $filename) {
    if($filename =~ /(.*)\/(.*)/) {
      if(!-d $1) {
	mkdir($1) || croak "Could not create $1 ($ERRNO)\n";
      }
    }
    open(TMP, ">", $filename) ||
      croak "Could not create $filename ($ERRNO)\n";
    close(TMP);
  }
  $list->readFile($filename);
  return $list;
}

# Write an HTML entry for the given key and namelist
# Namelist = namelist
# Prefix = Prefix to use for this entry
# Key = key represented by this entry (i.e. user, class)
# Polarity: 0 - check means present, unchecked means absent
#           1 - check means absent, unchecked means present
sub writeHtmlEntry {
  my ($namelist, $prefix, $key, $polarity) = @_;

  $prefix .= ".";
  my $oldPrefix = "old".ucfirst($prefix);
  my $state = $namelist->contains($key);
  if($polarity) {
    $state = ! $state;
  }

  hidden($oldPrefix . $key, $state);
  printCheckbox($prefix . $key, $state);
}

sub isEnabled {
  my ($namelist, $key, $polarity) = @_;
  my $result = $namelist->contains($key);
  if($polarity) {
    $result = !$result;
  }
  return $result;
}

sub enable {
  my ($namelist, $key, $polarity) = @_;
  if($polarity) {
    $namelist->remove($key);
  } else {
    $namelist->add($key);
  }
}


sub disable {
  my ($namelist, $key, $polarity) = @_;
  enable($namelist, $key, !$polarity);
}

# Update file from HTML reply
sub updateFileFromHtml {
  my ($namelist, $in, $prefix, $polarity) = @_;

  $prefix .= ".";
  my $oldPrefix = "old".ucfirst($prefix);
  my $changed = 0;

  foreach my $rqKey (keys %$in) {
    next unless $rqKey =~ /^$prefix(cl$validChars*)/;
    my $key = $1;
    my $oldKey = $oldPrefix.$1;
    my $state = $in->{$rqKey};
    my $oldState = $in->{$oldKey};
    if($state == 1 && !$oldState && !isEnabled($namelist, $key, $polarity)) {
      # enabled in WWW request, but disabled in namelist => enable it
      enable($namelist, $key, $polarity);
      $changed=1;
    }
  }

  foreach my $rqKey (keys %$in) {
    next unless $rqKey =~ /^$oldPrefix(cl$validChars*)/;
    my $key = $1;
    my $state = $in->{$rqKey};
    if($state == 1 &&
       !$in->{$prefix.$key} &&
       isEnabled($namelist,$key, $polarity)) {
      # used to be enabled, now disabled => disable it
      disable($namelist, $key, $polarity);
      $changed=1;
    }
  }

  $namelist->write();
  return $changed;
}

# FIXME: print tristate boxes
my $DENIED = "denied";
my $ALLOWED = "allowed";
my $DEFAULT = "default";

my $tristateLabels = [
		      [ $DENIED, $text{tristate_denied} ],
		      [ $DEFAULT, $text{tristate_default} ],
		      [ $ALLOWED, $text{tristate_allowed} ] ];

sub updateFilesFromTristate {
  my ($allowList, $denyList, $prefix, $in, $filter) = @_;
  my $changed;

  $prefix .= ".";
  my $oldPrefix = "old".ucfirst($prefix);
  foreach my $rqKey (keys %$in) {
    next unless $rqKey =~ /^$prefix($validChars*)/;
    my $key = $1;
    my $oldState = $in->{$oldPrefix . $key};
    my $newState = $in->{$prefix . $key};
    next if $oldState eq $newState;
    if(defined $filter) {
      next unless $filter->($key);
    }
    $changed=1;
    if($newState eq $ALLOWED) {
      $denyList->remove($key);
      $allowList->add($key);
    } elsif($newState eq $DENIED) {
      $denyList->add($key);
      $allowList->remove($key);
    } elsif($newState eq $DEFAULT) {
      $denyList->remove($key);
      $allowList->remove($key);
    }
  }

  $denyList->write();
  $allowList->write();
  return $changed;
}

sub updateUserFilesFromTristate {
  my ($prefix, $in, $filter) = @_;
  my $base = "$LLL_SECURITY_DIR/$prefix/user.";
  my $allowList = getNamelist($base . "allow");
  my $denyList = getNamelist($base . "deny");
  updateFilesFromTristate($allowList, $denyList, $prefix, $in, $filter);
}

sub getTriState {
  my ($allowList, $denyList, $object) = @_;

  my $allowState = $allowList->contains($object);
  my $denyState = $denyList->contains($object);

  my $state = "invalid";
  if($allowState && !$denyState) {
    $state = $ALLOWED;
  } elsif(!$allowState && $denyState) {
    $state = $DENIED;
  } elsif(!$allowState && !$denyState) {
    $state = $DEFAULT;
  }
}

sub printTriState {
  my ($allowList, $denyList, $prefix, $object, $printChoice) = @_;

  my $state = getTriState($allowList, $denyList, $object);

  $prefix .= ".";
  my $oldPrefix = "old".ucfirst($prefix);
  hidden("$oldPrefix$object", $state);
  if($printChoice) {
    printSelectionList("$prefix$object", $state, $tristateLabels);
  } else {
    print _getTriStateLabel($state);
  }
}

sub printUserTriState {
  my ($prefix, $object, $printChoice) = @_;
  my $base = "$LLL_SECURITY_DIR/$prefix/user.";
  my $allowList = getNamelist($base . "allow");
  my $denyList = getNamelist($base . "deny");
  printTriState($allowList, $denyList, $prefix, $object, $printChoice);
}

sub getTriStateLabel {
  my ($statePair, $object) = @_;
  my $state = getTriState($statePair->{allow}, $statePair->{deny}, $object);
  return _getTriStateLabel($state);
}

sub _getTriStateLabel {
  my ($state) = @_;

  foreach my $line (@$tristateLabels) {
    if($line->[0] eq $state) {
      return $line->[1];
    }
  }
  return "UNKNOWN";
}

sub getUserTriStatePair {
  my ($prefix) = @_;
  my $base = "$LLL_SECURITY_DIR/$prefix/user.";
  my $allowList = getNamelist($base . "allow");
  my $denyList = getNamelist($base . "deny");
  return { allow => $allowList, deny => $denyList };
}

1;
