#!/usr/bin/perl

# File with utility records to parse logfiles

use strict;

use Time::Local;
use English;

my $revMonths = {"Jan" =>  0,
		 "Feb" =>  1,
		 "Mar" =>  2,
		 "Apr" =>  3,
		 "May" =>  4,
		 "Jun" =>  5,
		 "Jul" =>  6,
		 "Aug" =>  7,
		 "Sep" =>  8,
		 "Oct" =>  9,
		 "Nov" => 10,
		 "Dec" => 11 };

my @now = localtime(time);
my $nowYearBase = [$now[5]+1900, $now[4], $now[3]];

sub shellEscape {
  my ($in) = @_;

  $in =~ 's/\\/\\\\/g';
  $in =~ 's/\"/\\\"/g';
  $in =~ 's/\$/\\\$/g';
  $in =~ 's/\!/\\\!/g';
  $in =~ 's/\`/\\\`/g';
  return $in;
}


sub logReadLoop(&$@) {
    my ($callback, $logfile, $filter, $minDate, $maxDate) = @_;
    my @ret;
    my ($yearbase, $begTick, $endTick);

    $filter = shellEscape($filter);
    foreach my $file (glob("$logfile-*"), "$logfile") {
      my $begTick = $endTick;
      if($file =~ /$logfile-(\d?\d?\d\d)(\d\d)(\d\d)(\.gz)?/) {
	my $year = $1;
	if ($year < 50) {
	  $year += 100; # 2000+, old notation
	} elsif ($year < 100) {
	  $year += 0; # 1900+, old notation
	} else {
	  $year -= 1900; # new notation
	}
	my $month = $2;
	my $day = $3;
	my $compressed = $4;

	$yearbase = [ $year, $month, $day ];
	$endTick = timelocal(0,0,0, $day, $month-1, $year);

	if($minDate > $endTick + 86399 || $maxDate < $begTick) {
	  next;
	}

	open(LOGFILE, "zcat $file $filter |") ||
	  &error(&text('logfile_uncompress',$file));
      } else {
	$yearbase = $nowYearBase;
	$endTick = time;

	if(($minDate && $minDate > $endTick + 86399) ||
	   ($maxDate && $maxDate < $begTick)) {
	  next;
	}
	open(LOGFILE, "<$file") ||
	  &error(&text('error_openRead', $file, $ERRNO));
      }

      while(<LOGFILE>) {
	chop;
	my $ret = &$callback($_, $yearbase);
	if (defined $ret) {
	  push (@ret, $ret);
	}
      }
      close(LOGFILE);
    }
    return @ret;
}


sub getMinMaxDates($) {
    my ($in) = @_;

    my ($searchMinDate, $searchMaxDate);

    if ($in->{'recent'} && $in->{'recent'} > 0) {
	$searchMinDate = time - $in->{'recent'};
	$searchMaxDate = time;
    } else {

	if($in->{'min.J'}) {
	    $searchMinDate = jmaToTime("min", $in);
	}

	if($in->{'max.J'}) {
	    $searchMaxDate = jmaToTime("max", $in) + 86399;
	}

	if ($searchMaxDate && $searchMinDate &&
	    $searchMaxDate < $searchMinDate) {
	    &error($text{logfile_badRange});
	}
    }

    return ($searchMinDate, $searchMaxDate);
}


sub parseSyslogDate($$@) {
    my ($line, $yearbase, $minDate, $maxDate) = @_;
    if($line =~ /^([A-Z][a-z]{2}) *(\d*) *(\d*):(\d{2}):(\d{2})\s*(.*)$/) {
	my $month      = $revMonths->{$1};
	my $day        = $2;
	my $year       = $yearbase->[0]; # fixme
	if($month > $yearbase->[1]) {
		$year--;
	} elsif($month == $yearbase->[1] && $day > $yearbase->[2]) {
		$year--;
	}
	my $hour       = $3;
	my $minute     = $4;
	my $sec        = $5;
	my $rest       = $6;

	my $date   = timelocal($sec, $minute, $hour, $day, $month, $year);
	if ($minDate && $date < $minDate) {
	    return undef;
	}

	if ($maxDate && $date > $maxDate) {
	    return undef;
	}
	return ($date, $rest);
    } else {
	return undef;
    }
}


my $months = ["Jan", "Feb", "Mar", "Avr", "Mai", "Jun",
	      "Jul", "Aou", "Sep", "Oct", "Nov", "Dec" ];

sub dateHeure($) {
    my ($date) = @_;

    my ($sec, $min, $hour, $mday, $mon, $year) = localtime($date);
    my $date = sprintf("%2d %s %s", $mday, $months->[$mon], $year + 1900);
    my $heure = sprintf("%02d:%02d:%02d", $hour, $min, $sec);

    return ($date, $heure);
}



sub printDateAndSubmit {
    my %date;

    %date=();
    &timeToJma(time(), \%date, "min");
    &timeToJma(time(), \%date, "max");

    &tableRow([{align=>'right'}, $text{logfile_from},
	       {align=>'left'}, sub {inputDate("min", \%date);},
	       {align=>'right'}, $text{logfile_to},
	       {align=>'left'}, sub {inputDate("max", \%date);}]);
    tableRow { submitLine(4) };
}
