use strict;
use vars qw($logged $whatfailed $config_directory %config);

use Sys::Syslog qw(:DEFAULT setlogsock);
use English;
use UserDBFile;

my $webminUser = $ENV{'REMOTE_USER'};
my $clientIp = $ENV{'REMOTE_HOST'};
my @webminUsers = ();

sub role_filename
{
    return "$config_directory/school/webmin.acl";
}

my $impliedBy = {
    stagiaire =>    [ "titulaire"],
    titulaire =>    [ "admin" ],
    installateur => [ "admin" ] };

sub getAllRoles {
    return ("admin", "titulaire", "stagiaire", "installateur");
}

my %access = get_module_acl();

sub haveRole($) {
    my $role = shift(@_);

    if ($access{$role}) {
      return 1;
    }

    foreach my $impRole (@{$impliedBy->{$role}}) {
	if (&haveRole($impRole)) {
	    return 1;
	}
    }

    return 0;
}

sub requireRole(@) {
    foreach my $role (@_) {
	if(haveRole $role) {
	    return;
	}
    }
    logMsg(text('logMsg_roleAssume',join(" ", @_), $whatfailed));
    &error(text('roles_noRole',$webminUser,
		       join("</em> ".$text{roles_or}." <em>",@_)));
}

# Operations permitted on your own account, or on the account of a student by
# a teacher
# Administrators can edit everybody
# Teachers can edit students, and themselves
# Everybody else may just edit themselves
#
# Applies to actions such as
#  editFiles
#  greenbutton
sub _amSelfOrStudent {
    my ($role, $line) = @_;

    if ($webminUser eq $line->[0]) {
	# every user can modify his own settings
	return 1;
    }

    if ($line->isStudent) {
	return haveRole $role;
    }

    return haveRole "admin";
}

sub _requireSelfOrStudent {
    if (_amSelfOrStudent(@_)) {
	return;
    }

    &logMsg(&text('logMsg_roleAction',$whatfailed));
    &error(&text('roles_noAccess', $webminUser));
}

sub amSelfOrStudent($) {
  return _amSelfOrStudent("titulaire", @_);
}

sub requireSelfOrStudent($) {
  return _requireSelfOrStudent("titulaire", @_);
}

sub amSelfOrStudentGb($) {
  return _amSelfOrStudent("stagiaire", @_);
}

sub requireSelfOrStudentGb($) {
  return _requireSelfOrStudent("stagiaire", @_);
}


# Administrators can edit all users' rights
# Teachers can edit student users' rights'
# All others can not edit rights

# This governs actions such as
#   - put on hold/reactivate
#   - user's right management (features)
sub canEditRights($) {
    my $user = shift(@_);
    if(haveRole('admin')) {
      return 1;
    }
    if(haveRole('titulaire')) {
      if(!ref($user)) {
	$user = userDB->get($user);
      }
      if($user->isStudent) {
	return 1;
      }
    }
    return 0;
}

END {
    if (!$logged) {
	my $msg = "";
	if(defined %in) {
	    my ($key, $value);
	    while (($key, $value) = each %in) {
		if ($key eq "action" && $key eq "$text{button_submit}") {
		    next;
		}
		if ($value ne "" && $key ne "pass1" && $key ne "pass2") {
		    $msg .= " $key=$value";
		}
	    }
	    $msg =~ s/\n/ \\ /;
	    $msg =~ s/\r/ \\ /;
	}
	logMsg($msg);
    }
    closelog();
};

my $webminFile;

sub logMsg {
    my ($message) = @_;

    $logged = 1;
    if($config{logging}) {
	syslog('info', "%s/%s: %s %s",
	       $webminUser, $clientIp, $webminFile, $message);
    }
}

sub logError {
    my ($message) = @_;

    logMsg($message);
    &error($message);
}

sub getWebminUser {
	return $webminUser;
}

#package main;
use vars qw($demo_allow $whatfailed);
use Sys::Syslog qw(:DEFAULT setlogsock);


setlogsock('unix');
openlog("webmin", "pid", "user");

$webminFile=(caller(1))[1];
if($webminFile =~ m(^.*/([^/]*)\.cgi)) {
  $webminFile = $1;
}

if(($webminFile =~ /Act$/) && $access{demo} && ($demo_allow ne "1")) {
  $logged = 1;
  &error($text{roles_demo});
}

1;
